#include "GstPlayer.h"
#include <iostream>
#include <fstream>

/**
 * Create instance of GstPlayer for playing a video
 * @param play_unaccelerated	true if you should play with acceleration, false otherwise
 */
GstPlayer::GstPlayer()
{
	mode = UNACCELERATED;
	gst_init(NULL, NULL);
	init_pipeline();
}

/**
 * Destroy instance of GstPlayer
 */
GstPlayer::~GstPlayer()
{
}

/**
 * Play video
 */
void
GstPlayer::play()
{
	gst_element_set_state(pipeline, GST_STATE_PLAYING);
}

/**
 * Pause video
 */
void
GstPlayer::pause()
{
	gst_element_set_state(pipeline, GST_STATE_PAUSED);
}

/**
 * Stop video
 */
void
GstPlayer::stop()
{
	destroy();
	init_pipeline();
}

/**
 * Destroy this instance of GstPlayer by removing all references to components.
 * Necessary step in some cases before starting the stream after a STOP, so
 * this should be called whenever restarting the stream
 */
void
GstPlayer::destroy()
{
	gst_element_set_state(pipeline, GST_STATE_NULL);
	gst_object_unref(pipeline);
	gst_object_unref(bus);
}

/**
 * Provide reference to pipeline bus for monitoring pipe status
 * @return	reference to pipeline bus
 */
GstBus*
GstPlayer::get_bus()
{
	return bus;
}


/**
 * Initialize pipeline based on configuration file
 */
void
GstPlayer::init_pipeline()
{
	const int MAX_LENGTH = 500;
	char pipeline_string[MAX_LENGTH];

	// Retrieve pipeline configuration
	if (mode == ACCELERATED)
		get_config("pipeline_fast", pipeline_string);
	else if (mode == UNACCELERATED)
		get_config("pipeline", pipeline_string);
	else
		get_config("pipeline_camera", pipeline_string);

	printf("Pipeline string is: %s\n", pipeline_string);

	// Create pipeline from configuration string
	pipeline = gst_parse_launch(pipeline_string, NULL);

	// Retrieve pipeline signal bus
	bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline));

	// Set pipeline to ready
	gst_element_set_state(pipeline, GST_STATE_READY);

}

/**
 * Set whether this video should play with acceleration or not
 * By default, this resets the pipeline, so do not call this midstream.
 * @param newMode	new mode of play, whether accelerated, nonaccelerated,
 * 	camera, or other
 */
void
GstPlayer::set_mode(unsigned int newMode)
{
	mode = newMode;
	stop();
}
